#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 19-11-26 上午10:52
# @Author  : Bai Yun
# @File    : command_models.py
# @Software: PyCharm Community Edition
# subcommand: models
import sys
sys.path.append("../../")
from . import constant
from . import util
from . import model_test


def models_main_command(main_subparser):
  models_parser = main_subparser.add_parser(
      "models", help="Commands about models")
  models_subparser = models_parser.add_subparsers(
      dest="models_command", help="Subcommands of models")
  models_list_command(models_subparser)
  models_deploy_command(models_subparser)
  models_describe_command(models_subparser)
  models_create_command(models_subparser)
  models_update_command(models_subparser)
  models_delete_command(models_subparser)
  models_start_command(models_subparser)
  models_ready_command(models_subparser)
  models_roll_command(models_subparser)
  models_upload_command(models_subparser)
  models_download_command(models_subparser)
  models_logs_command(models_subparser)
  models_metrics_command(models_subparser)
  models_predict_command(models_subparser)
  models_events_command(models_subparser)
  models_test_command(models_subparser)
  models_review_command(models_subparser)
  models_daildown_command(models_subparser)
  models_onboard_command(models_subparser)
  models_restart_command(models_subparser)


def models_list_command(models_subparser):
  models_list_parser = models_subparser.add_parser(
      "list", help="List model services")
  models_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                  choices=constant.CLOUDML_PRIORITY_CLASSES,
                                  help="The priority class to list jobs with. If not set, will return all.")
  models_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                                  help="The page number of the list.")
  models_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                  help="The page size of the list.")
  models_list_parser.set_defaults(func=util.list_models)


def models_deploy_command(models_subparser):
  models_deploy_parser = models_subparser.add_parser(
      "deploy", help="deploy model service with json-format config file")
  models_deploy_parser.add_argument(
      "-jf",
      "--deploy_config",
      dest="deploy_config",
      help="The path of the json-format config file",
      required=True)
  models_deploy_parser.set_defaults(func=util.deploy_models)


def models_describe_command(models_subparser):
  # subcommand of models: describe
  models_describe_parser = models_subparser.add_parser(
      "describe", help="Describe the model service")
  models_describe_parser.add_argument(
      "model_name", help="The name of the model")
  models_describe_parser.add_argument(
      "model_version", help="The version of the model")
  models_describe_parser.set_defaults(func=util.describe_model)


def models_create_command(models_subparser):
  # subcommand of models: create
  models_create_parser = models_subparser.add_parser(
      "create", help="Create model service")
  models_create_parser.add_argument(
      "-n",
      "--model_name",
      dest="model_name",
      help="The name of the model",
      required=True)
  models_create_parser.add_argument(
      "-v",
      "--model_version",
      dest="model_version",
      help="The version of the model",
      required=True)
  models_create_parser.add_argument(
      "-u",
      "--model_uri",
      dest="model_uri",
      help="The uri of the model",
      required=False)
  models_create_parser.add_argument(
      "-t",
      "--service_type",
      dest="service_type",
      default="test",
      help="The value is online or test"
  )
  models_create_parser.add_argument(
      "-dt",
      "--deploy_type",
      dest="deploy_type",
      default="ingress",
      help="The value is ingress, pod-dns"
  )
  models_create_parser.add_argument(
      "-a", "--model_args", dest="model_args", help="The string of args")
  models_create_parser.add_argument(
      "-fe", "--fds_endpoint", dest="fds_endpoint", help="The fds endpoint use")
  models_create_parser.add_argument(
      "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket to mount to /fds")
  models_create_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The CPU limit with unit core")
  models_create_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit with unit M or G")
  models_create_parser.add_argument(
      "-g", "--gpu_limit", dest="gpu_limit", help="The number of GPU limit")
  models_create_parser.add_argument(
      "-gm", "--gpu_memory", dest="gpu_memory", help="The required least GPU memory")
  gpu_type_help = "select gpu type from ({})".format("/".join(constant.GPULIST))
  if not constant.ENABLE_GPU_TYPE_SELECT:
    gpu_type_help = "[DEPRECATED] " + gpu_type_help
  models_create_parser.add_argument(
      "-gt",
      "--gpu_type",
      dest="gpu_type",
      help=gpu_type_help
  )
  models_create_parser.add_argument(
      "-R",
      "--enable_rank",
      dest="enable_rank",
      action="store_true",
      help="Create priority high models"
  )
  models_create_parser.add_argument(
      "-d", "--docker_image", dest="docker_image", help="The docker image")
  models_create_parser.add_argument(
      "--debug",
      dest="debug",
      action="store_true",
      help="debug with tensorflow Serving",
  )
  models_create_parser.add_argument(
      "-dc",
      "--docker_command",
      dest="docker_command",
      help="The docker command")
  models_create_parser.add_argument(
      "-F",
      "--framework",
      dest="framework",
      help="The framework of machine learning")
  models_create_parser.add_argument(
      "-V",
      "--framework_version",
      dest="framework_version",
      help="The version of the framework")
  models_create_parser.add_argument(
      "-r", "--replicas", dest="replicas", help="The num of replicas")
  models_create_parser.add_argument(
      "-pc",
      "--prepare_command",
      dest="prepare_command",
      help="The prepare command")
  models_create_parser.add_argument(
      "-fc",
      "--finish_command",
      dest="finish_command",
      help="The finish command")
  models_create_parser.add_argument(
      "-nsk",
      "--node_selector_key",
      dest="node_selector_key",
      help="The node selector key")
  models_create_parser.add_argument(
      "-nsv",
      "--node_selector_value",
      dest="node_selector_value",
      help="The node selector value")
  models_create_parser.add_argument(
      "-W", "--watch", action="store_true", help="Watch the status of model creation")
  models_create_parser.add_argument(
      "-sv",
      "--save_mode",
      dest="save_mode",
      help="Save model, don't run",
      action = 'store_true'
      )
  models_create_parser.add_argument(
      "--use_seldon",
      dest="use_seldon",
      help="Using seldon service",
      action="store_true"
  )
  models_create_parser.add_argument(
      "--engine_cpu",
      dest="engine_cpu",
      help="The CPU limit with unit core"
  )
  models_create_parser.add_argument(
      "--engine_memory",
      dest="engine_mem",
      help="The memory limit with unit M or G"
  )
  models_create_parser.add_argument(
      "--run_class_name",
      dest="run_class_name",
      help="Load model class name"
  )
  models_create_parser.add_argument(
      "--service_schema",
      dest="service_schema",
      help="Service schema (REST or GRPC)",
      default="GRPC"
  )
  models_create_parser.add_argument(
      "--initial_delay_sec",
      dest="initial_delay_sec",
      help="Initial Delay Seconds"
  )
  models_create_parser.add_argument(
      "--pod_replicas",
      dest="pod_replicas",
      help="The num of pod replicas"
  )
  models_create_parser.add_argument(
      "--multi_serving_name",
      dest="multi_serving_name",
      help="The name of the multi serving"
  )
  models_create_parser.add_argument(
      "--per_process_gpu_memory_fraction",
      dest="per_process_gpu_memory_fraction",
      help="Fraction that each process occupies of the GPU memory space the value is between 0.0 and 1.0 (with 0.0 as the default) If 1.0, "
           "the server will allocate all the memory when the server starts, If 0.0, Tensorflow will automatically select a value"
  )
  models_create_parser.add_argument(
      "--describe",
      dest="describe",
      help="describe for model service"
  )
  models_create_parser.add_argument(
      "-tel",
      "--telephone",
      dest="telephone",
      help="The value of oncall phone number!"
  )
  models_create_parser.add_argument(
      "--version_policy",
      dest="version_policy",
      help="The value of update version policy,default latest. (latest/all/specific)"
  )
  models_create_parser.add_argument(
      "-argo",
      "--argo_mode",
      dest="argo_mode",
      help="mark it a argo job",
      action='store_true')

  models_create_parser.add_argument(
    "-mf",
    "--model_format",
    dest="model_format",
    default="savedmodel",
    help="The value is savedmodel, multi-sm, tf-trt, others")

  models_create_parser.add_argument(
    "-e",
    "--owner_email",
    dest="owner_email",
    help="The value is owner email."
  )
  models_create_parser.add_argument(
    "--priority_class",
    dest="priority_class",
    choices=constant.CLOUDML_PRIORITY_CLASSES,
    default=constant.MS_DEFAULT_PRIORITY_CLASS,
    help="The value of priority_class."
  )
  models_create_parser.add_argument(
    "--grace_period_seconds",
    dest="grace_period_seconds",
    help="The value of grace_period_seconds"
  )
  models_create_parser.add_argument(
    "--readiness_command",
    dest="readiness_command",
    help="The value of readiness_command."
  )
  models_create_parser.add_argument(
    "--meta_info",
    dest="meta_info",
    help="The value of meta_info."
  )
  models_create_parser.set_defaults(func=util.create_model)


def models_update_command(models_subparser):
  # subcommand of models: update
  models_update_parser = models_subparser.add_parser(
      "update", help="Update the model service")
  models_update_parser.add_argument(
      "model_name", help="The name of the model")
  models_update_parser.add_argument(
      "model_version", help="The version of the model")

  models_update_parser.add_argument(
      "-w",
      "--weight",
      dest="weight",
      help="the weight of how much traffic go to this version"
  )

  models_update_parser.add_argument(
      "-u",
      "--model_uri",
      dest="model_uri",
      help="The uri of the model"
  )
  models_update_parser.add_argument(
      "-r",
      "--replicas",
      dest="replicas",
      help="The num of replicas")
  models_update_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The CPU limit with unit core")
  models_update_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit with unit M or G")
  models_update_parser.add_argument(
      "-g",
      "--gpu_limit",
      dest="gpu_limit",
      help="The number of GPU limit")
  models_update_parser.add_argument(
      "-F",
      "--framework",
      dest="framework",
      help="The framework of machine learning")
  models_update_parser.add_argument(
      "-p",
      "--port",
      dest="port",
      help="Reset Model service port"
  )
  models_update_parser.add_argument(
      "--engine_cpu",
      dest="engine_cpu",
      help="The CPU limit with unit core"
  )
  models_update_parser.add_argument(
      "--engine_memory",
      dest="engine_mem",
      help="The memory limit with unit M or G"
  )
  models_update_parser.add_argument(
      "--meta_info",
      dest="meta_info",
      help="The meta info of the model service"
  )
  models_update_parser.set_defaults(func=util.update_model)


def models_delete_command(models_subparser):
  # subcommand of models: delete
  models_delete_parser = models_subparser.add_parser(
      "delete", help="Delete the model service")
  models_delete_parser.add_argument("model_name", help="The name of the model")
  models_delete_parser.add_argument(
      "model_version", help="The version of the model")
  models_delete_parser.add_argument(
      "--history",
      dest="history",
      action="store_true",
      help="Delete the model service when state is historical."
  )
  models_delete_parser.set_defaults(func=util.delete_model)


def models_start_command(models_subparser):
  # subcommand of models: start
  models_start_parser = models_subparser.add_parser(
      "start", help="Start the model service")
  models_start_parser.add_argument(
      "model_name", help="The name of the model")
  models_start_parser.add_argument(
    "model_version", help="The version of the model")
  models_start_parser.set_defaults(func=util.start_model)


def models_ready_command(models_subparser):
  # subcommand of models: ready
  models_ready_parser = models_subparser.add_parser(
      "ready", help="judge the readiness of the model service")
  models_ready_parser.add_argument(
      "model_name", help="The name of the model")
  models_ready_parser.add_argument(
      "model_version", help="The version of the model")
  models_ready_parser.set_defaults(func=util.ready_model)


def models_roll_command(models_subparser):
  # subcommand of models: roll
  models_roll_parser = models_subparser.add_parser(
      "roll", help="rolling update of the model service")
  models_roll_parser.add_argument(
      "model_name", help="The name of the model")
  models_roll_parser.add_argument(
      "-fv",
      "--from_version",
      dest="from_version",
      help="The source version of the model"
  )
  models_roll_parser.add_argument(
      "-tv",
      "--to_version",
      dest="to_version",
      help="The destination version of the model"
  )
  models_roll_parser.add_argument(
      "-ut",
      "--update_timeout",
      dest="update_timeout",
      help="wait period by second of every updating turn"
  )
  models_roll_parser.set_defaults(func=util.roll_model)


def models_upload_command(models_subparser):
  # subcommand of models: upload
  models_upload_parser = models_subparser.add_parser(
      "upload", help="upload model data")
  models_upload_parser.add_argument(
      "local_model_path", help="The local path of the model")
  models_upload_parser.add_argument(
      "-n",
      "--model_name",
      dest="model_name",
      help="The name of the model",
      required=False)
  models_upload_parser.add_argument(
      "-v",
      "--model_version",
      dest="model_version",
      help="The version of the model",
      required=False)
  models_upload_parser.add_argument(
      "-at",
      "--archive_type",
      dest="archive_type",
      default="raw",
      help="The archive_type of the model files, raw or targz")

  models_upload_parser.set_defaults(func=util.upload_model_data)


def models_download_command(models_subparser):
  # subcommand of models: download
  models_download_parser = models_subparser.add_parser(
      "download", help="download model data")
  models_download_parser.add_argument(
      "local_model_path", help="The local path of the model")
  models_download_parser.add_argument(
      "-n",
      "--model_name",
      dest="model_name",
      help="The name of the model",
      required=True)
  models_download_parser.add_argument(
      "-v",
      "--model_version",
      dest="model_version",
      help="The version of the model",
      required=True)
  models_download_parser.add_argument(
      "-at",
      "--archive_type",
      dest="archive_type",
      default="raw",
      help="The archive_type of the model files, raw or targz")
  models_download_parser.set_defaults(func=util.download_model_data)


def models_logs_command(models_subparser):
  # subcommand of models: logs
  models_logs_parser = models_subparser.add_parser(
      "logs", help="Get the logs of the model service")
  models_logs_parser.add_argument(
      "model_name", help="The name of the model")
  models_logs_parser.add_argument(
      "model_version", help="The version of the model")
  models_logs_parser.add_argument(
      "-ri",
      "--replica_index",
      dest="replica_index",
      help="The replica index"
  )
  models_logs_parser.add_argument(
      "-p",
      "--proxy",
      dest="proxy_logs",
      action="store_true",
      help="Get the logs of the proxy"
  )
  models_logs_parser.set_defaults(func=util.get_model_logs)


def models_metrics_command(models_subparser):
  # subcommand of models: metrics
  models_metrics_parser = models_subparser.add_parser(
      "metrics", help="Get the metrics of the model service")
  models_metrics_parser.add_argument("model_name", help="The name of the model")
  models_metrics_parser.add_argument(
      "model_version", help="The version of the model")
  models_metrics_parser.set_defaults(func=util.get_model_metrics)


def models_predict_command(models_subparser):
  # subcommand of models: predict
  models_predict_parser = models_subparser.add_parser(
      "predict", help="Request the model service and predict")
  models_predict_parser.add_argument(
      "-n",
      "--model_name",
      dest="model_name",
      help="The name of the model",
      required=True)
  models_predict_parser.add_argument(
      "-v",
      "--model_version",
      dest="model_version",
      help="The version of the model")
  models_predict_parser.add_argument(
      "-s", "--server", dest="server", help="The address of the server")
  models_predict_parser.add_argument(
      "-f",
      "--filename",
      dest="filename",
      help="The json data file",
      required=True)
  models_predict_parser.add_argument(
      "-t", "--timeout", dest="timeout", help="The timeout of gRPC request")
  models_predict_parser.set_defaults(func=util.do_predict)


def models_events_command(models_subparser):
  # subcommand of models: events
  models_events_parser = models_subparser.add_parser(
      "events", help="Get the events of the model service")
  models_events_parser.add_argument(
      "model_name", help="The name of the model service")
  models_events_parser.add_argument(
      "model_version", help="The version of the model service")
  models_events_parser.set_defaults(func=util.get_model_service_events)


def models_test_command(models_subparser):
  # subcommand of models: test
  models_test_parser = models_subparser.add_parser("test", help="Test and tuning model service.")

  models_test_subparsers = models_test_parser.add_subparsers(dest="models_test_command", help="Subcommands of models test")

  """
  models_test_config_parser = models_test_subparsers.add_parser("config", help="Config test env.")
  models_test_config_parser.add_argument("--testcase_file", dest="testcase_file", help="The name of the model service")
  models_test_config_parser.set_defaults(func=util.config_model_service_test_env)
  """

  models_test_start_parser = models_test_subparsers.add_parser("start", help="Start to run test cases.")
  models_test_start_parser.add_argument("--testcase_file", dest="testcases", help="File contains test cases.")
  models_test_start_parser.set_defaults(func=model_test.start_model_service_test_cases)

  models_test_result_parser = models_test_subparsers.add_parser("result", help="Show or save test result.")
  models_test_result_parser.add_argument("handle", help="value is show or save.")
  models_test_result_parser.add_argument("--result_file", dest="result_file", help="save result to file.")
  models_test_result_parser.set_defaults(func=model_test.handle_model_service_test_result)

  models_test_clear_parser = models_test_subparsers.add_parser("clear", help="Clear model services and test case data.")
  models_test_clear_parser.set_defaults(func=model_test.clear_model_service_test_data)


def models_review_command(models_subparser):
  models_review_parser = models_subparser.add_parser(
      "review", help="Review of model pre prod, test and approve")
  models_review_parser.add_argument(
      "model_name", help="The name of the model service")
  models_review_parser.add_argument(
      "model_version", help="The version of the model service")
  models_review_parser.add_argument(
      "review_leader", help="The leader who will review the apply")
  models_review_parser.set_defaults(func=util.model_review)


def models_daildown_command(models_subparser):
  models_daildown_parser = models_subparser.add_parser(
      "daildown", help="Dail Down the model")
  models_daildown_parser.add_argument(
      "model_name", help="The name of the model service")
  models_daildown_parser.add_argument(
      "model_version")
  models_daildown_parser.set_defaults(func=util.model_daildown)


def models_onboard_command(models_subparser):
  models_onboard_parse = models_subparser.add_parser(
      "onboard", help="Formal Onboard model service")
  models_onboard_parse.add_argument(
      "model_name", help="The name of model service")
  models_onboard_parse.add_argument(
      "model_version", help="The version of model service")
  models_onboard_parse.set_defaults(func=util.model_onboard)


def models_restart_command(models_subparser):
  models_restart_parse = models_subparser.add_parser(
      "restart", help="Restart model service instart."
  )
  models_restart_parse.add_argument(
      "model_name", help="The name of model service."
  )
  models_restart_parse.add_argument(
      "model_version", help="The version of model service."
  )
  models_restart_parse.set_defaults(func=util.restart_model_service_instance)
