from __future__ import print_function

import time
import yaml
import json

from cloud_ml_sdk.command.util import print_model_service_info
from cloud_ml_sdk.command.util import exit_as_error

from cloud_ml_sdk.models.model_service import ModelService

from cloud_ml_sdk.client import CloudMlClient

import logging
logging.getLogger("requests").setLevel(logging.WARNING)


# print msgs with end as '\r'
def cr_print(*msgs):
    import sys
    print(*msgs, end='\r')
    sys.stdout.flush()


# model service test
def config_model_service_test_env(args):
    print ("pass")


def model_test_create_model_service(testcase):
    try:
        client = CloudMlClient()
    except Exception as e:
        exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
        return
  
    model = ModelService(testcase['model_name'], testcase['model_version'], testcase['model_uri'])

    # if testcase.has_key('service_type'):
    if 'service_type' in testcase:
        model.model_type = testcase['service_type']

    # if testcase.has_key('fds_endpoint'):
    if 'fds_endpoint' in testcase:
        model.fds_endpoint = testcase['fds_endpoint']
    elif client.fds_endpoint:
        model.fds_endpoint = client.fds_endpoint

    # if testcase.has_key('fds_bucket'):
    if 'fds_bucket' in testcase:
        model.fds_bucket = testcase['fds_bucket']
    elif client.fds_bucket:
        model.fds_bucket = client.fds_bucket

    # if testcase.has_key('model_args'):
    if 'model_args' in testcase:
        model.model_args = testcase['model_args']

    # if testcase.has_key('cpu_limit'):
    if 'cpu_limit' in testcase:
        model.cpu_limit = str(testcase['cpu_limit'])

    # if testcase.has_key('memory_limit'):
    if 'memory_limit' in testcase:
        model.memory_limit = testcase['memory_limit']

    # if testcase.has_key('gpu_limit'):
    if 'gpu_limit' in testcase:
        model.gpu_limit = testcase['gpu_limit']

    # if testcase.has_key('gpu_memory'):
    if 'gpu_memory' in testcase:
        model.gpu_memory = testcase['gpu_memory']

    # if testcase.has_key('framework'):
    if 'framework' in testcase:
        model.framework = testcase['framework']

    # if testcase.has_key('framework_version'):
    if 'framework_version' in testcase:
        model.framework_version = testcase['framework_version']

    # if testcase.has_key('docker_image'):
    if 'docker_image' in testcase:
        model.docker_image = testcase['docker_image']

    # if testcase.has_key('docker_command'):
    if 'docker_command' in testcase:
        model.docker_command = testcase['docker_command']

    # if testcase.has_key('replicas'):
    if 'replicas' in testcase:
        model.replicas = testcase['replicas']

    # if testcase.has_key('prepare_command'):
    if 'prepare_command' in testcase:
        model.prepare_command = testcase['prepare_command']

    # if testcase.has_key('finish_command'):
    if 'finish_command' in testcase:
        model.finish_command = testcase['finish_command']

    # if testcase.has_key('node_selector_key'):
    if 'node_selector_key' in testcase:
        model.node_selector_key = testcase['node_selector_key']

    # if testcase.has_key('node_selector_value'):
    if 'node_selector_value' in testcase:
        model.node_selector_value = testcase['node_selector_value']

    # if testcase.has_key('gpu_type'):
    if "gpu_type" in testcase:
        # if testcase.has_key('gpu_limit'):
        if 'gpu_limit' in testcase:
            model.gpu_type = testcase['gpu_type']
        else:
            raise ValueError("ERROR: If you want select gpu type, gpu limit (-g) cannot be None.")

    # if testcase.has_key('enable_rank'):
    #     model.enable_rank = 1
    # else:
    #     model.enable_rank = 0

    # if args.debug:
    #     model.debug = 1
    # else:
    #     model.debug = 0

    # if args.watch:
    #     model_name = args.model_name
    #     model_version = args.model_version

    # if testcase.has_key('save_mode'):
    if 'save_mode' in testcase:
        model.save_mode = testcase['save_mode']

    # if args.use_seldon:
    #     model.use_seldon = args.use_seldon

    # if args.engine_cpu:
    #     model.engine_cpu = args.engine_cpu

    # if args.engine_mem:
    #     model.engine_mem = args.engine_mem

    # if args.run_class_name:
    #     model.run_class_name = args.run_class_name

    # if args.service_schema:
    #     model.service_schema = args.service_schema

    # if args.initial_delay_sec:
    #     model.initial_delay_sec = args.initial_delay_sec

    # if args.pod_replicas:
    #     model.pod_replicas = args.pod_replicas

    # model.multi_serving_name = args.multi_serving_name

    # if testcase.has_key('per_process_gpu_memory_fraction'):
    if 'per_process_gpu_memory_fraction' in testcase:
        if not testcase['multi_serving_name']:
            raise ValueError("ERROR: If you want share GPU with multi-serving, must be set multi_serving_name")
        model.per_process_gpu_memory_fraction = testcase['per_process_gpu_memory_fraction']

    # if args.describe:
    #     model.describe = args.describe

    # if args.telephone:
    #     model.telephone = args.telephone

    # if args.version_policy:
    #     model.version_policy = args.version_policy

    # if testcase.has_key('model_format'):
    if 'model_format' in testcase:
        model.model_format = testcase['model_format']
        

    model.owner_email = client.org_mail
    model.deploy_type = "ingress"

    response = client.create_model_service(model)

    if not isinstance(response, str):
        print_model_service_info(response)
        # if args.watch:
        #     print("\nThe model is creating, feel free to Ctrl+C to stop watching\n")
        #     print("{:32} {:4} {:16} {:32} {:32}".format(
        #             "Model_NAME",
        #             "VERSION",
        #             "STATE",
        #             "CREATED",
        #             "UPDATED"))
        #     while True:
        #         watch_response = client.describe_model_service(model_name, model_version)
        #         if not isinstance(watch_response, str):
        #             print("{:32} {:4} {:16} {:32} {:32}".format(
        #                     watch_response["model_name"],
        #                     watch_response["model_version"],
        #                     color_util.colorize_state(watch_response["state"]),
        #                     watch_response["create_time"],
        #                     watch_response["update_time"]))
        #         if watch_response["state"] == constant.MODEL_STATE_RUNNING:
        #             return
        #         try:
        #             time.sleep(constant.JOB_WATCH_INTERVAL)
        #         except KeyboardInterrupt:
        #             return
        #         else:
        #             return
    else:
        print("response: {}".format(response))


def model_test_get_testcase_info(testcase_id):
    try:
        client = CloudMlClient()
    except Exception as e:
        exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
        return
    
    response = client.get_model_test_case(testcase_id)

    return response


def model_test_create_pressure_task(testcase):
    #check if model service exist.

    #post test case to cloudml server.
    try:
        client = CloudMlClient()
    except Exception as e:
        exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
        return
    
    data = json.dumps(testcase)

    response = client.create_model_test_case(data)

    print(response)
    testcase_id = response['id']

    # wait till pressure task is finished.
    # task need 30s to run at least.
    print('send pressure and wait...')
    time.sleep(30)
    while True:
        # request to get test case info.
        testcase_info = model_test_get_testcase_info(testcase_id)
        if testcase_info['is_excuted'] == 1:
            print(testcase_info)
            break

        cr_print("waiting task finished...")
        # print("waiting task finished...", end='\r')
        time.sleep(2)


def model_test_delete_model_service(testcase):
    try:
        client = CloudMlClient()
    except Exception as e:
        exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
        return

    response = client.delete_model_service(testcase['model_name'], testcase['model_version'])
    print(response)
    
    time.sleep(3)

    while True:
        cr_print("start to check model status...")
        # print("start to check model status...", end='\r')
        try:
            client = CloudMlClient()
        except Exception as e:
            exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
            return
        
        response = client.describe_model_service(testcase['model_name'], testcase['model_version'])
        if isinstance(response, str):
            if "doesn't exist" in response:
                print("model is deleted.                        ")
                break
        
        time.sleep(2)


def model_test_wait_model_service_running(testcase):
    ok = False
    res = ''
    while True:
        try:
            client = CloudMlClient()
        except Exception as e:
            err = "ERROR: Failed to init CloudML Client: {}".format(e)
            exit_as_error(err)
            return False, err

        response = client.describe_model_service(testcase['model_name'], testcase['model_version'])

        # print "model_test_wait_model_service_running: desc model service: %s" % response
        
        if not isinstance(response, str):
            #print_model_service_info(response)
            if response['state'] == 'running':
                print("model service created successfully")
                ok = True
                break

            # print("model service is creating...", end='\r')
            cr_print("model service is creating...")
            time.sleep(3)
            continue
        
        else:
            err = "response: {}".format(response)
            print(err)
            ok = False
            res = err
            break
    return ok, res


def start_model_service_test_cases(args):

    # check env is stage or not.
    client = CloudMlClient()
    # if 'stag' not in client._endpoint and 'stress' not in client._endpoint:
    #     print 'only allowed in stage environment: https://staging-cloud-ml.api.xiaomi.net'
    #     return
    logging.getLogger("urllib3").setLevel(logging.WARNING)

    print (args.testcases)
    try:
        with open(args.testcases, "r") as fp:
            text = fp.read()
    except Exception as e:
        exit_as_error("ERROR: Failed to read {}, {}.".format(args.testcases, e))
        return
    
    testcases = yaml.safe_load(text)
    print (testcases)
    # send test case info to cloudml server.
    # server create model service and pressure by loop.
    # after each pressure, save result to database.
    for testcase in testcases:
        
        if testcase['is_enable'] != 1:
            continue

        # Remove None field and blank field
        for key in list(testcase.keys()):
            if testcase[key] is None or testcase[key] == '':
                del testcase[key]

        # check whether or not create model service
        if 'model_cluster' not in testcase and 'client_path' not in testcase:
            # post model service and wait done.

            # model service and client path are not assigned, model service need to be created
            model_test_create_model_service(testcase)
            # check whether model service is running only when model service is created by test
            ok, err = model_test_wait_model_service_running(testcase)
            if ok != True:
                return
        elif 'model_cluster' in testcase and 'client_path' in testcase:
            # both model service and client path are assigned
            pass
        # only one of model_cluster and client_path is filled
        else:
            # model service or client path is not assigned
            print("Both model_cluster and client_path need to be assigned")
            continue

        time.sleep(1)


        # post pressure task and wait done.
        model_test_create_pressure_task(testcase)

        if 'model_cluster' not in testcase and 'client_path' not in testcase:
            # delete model service only when model service is created by test
            model_test_delete_model_service(testcase)

    logging.getLogger("urllib3").setLevel(logging.DEBUG)


def get_model_test_result():
    try:
        client = CloudMlClient()
    except Exception as e:
        err = "ERROR: Failed to init CloudML Client: {}".format(e)
        exit_as_error(err)
        return None

    response = client.get_model_test_cases_by_org_id()

    if isinstance(response, dict):
        if response['Status'] == 'No test case found.':
            print('No test case info.')
            return None
    
    return response


def format_testcase_result(testcases):
    if None == testcases:
        return "No test result found."

    res = ''
    for testcase in testcases:
        # Bytes to MegaBytes
        perf_mem = float(testcase['perf_mem']) / (1024 * 1024)
        texts = [
            '\n-------------test case: %d -------------\n' % testcase['id'],
            'name:                     %s\n' % testcase['name'],
            'description:              %s\n' % testcase['description'],
            'model_name:               %s\n' % testcase['model_name'],
            'model_version:            %s\n' % testcase['model_version'],
            'pressure_user_num:        %s\n' % testcase['pressure_user_num'],
            'pressure_hatch_rate:      %s\n' % testcase['pressure_hatch_rate'],
            'perf_request_num:         %s\n' % testcase['perf_request_num'],
            'perf_qps:                 %s\n' % testcase['perf_qps'],
            'perf_ave:                 %s\n' % testcase['perf_ave'],
            'perf_p95:                 %s\n' % testcase['perf_p95'],
            'perf_p99:                 %s\n' % testcase['perf_p99'],
            'perf_cpu:                 %s\n' % testcase['perf_cpu'],
            'perf_mem:                 %sM\n' % perf_mem,
            'perf_gpu:                 %s\n' % testcase['perf_gpu'],
            'perf_gpu_mem_allocated:   %s\n' % testcase['perf_gpu_mem_allocated'],
            'perf_gpu_mem:             %s\n' % testcase['perf_gpu_mem'],
        ]

        res = res + ''.join(texts)
    
    return res


def handle_model_service_test_result(args):
    """
    get all testcase info by org_id.
    """
    if args.handle == 'show':
        response = get_model_test_result()
        text = format_testcase_result(response)
        print(text)

    elif args.handle == 'save':
        response = get_model_test_result()
        text = format_testcase_result(response)
        print(text)
        file_path = args.result_file
        try:
            with open(file_path, 'w') as fp:
                fp.write(text)
        except IOError as e:
            print(str(e))


def clear_model_service_test_data(args):
    """
    delete all testcase by org_id.
    """
    try:
        client = CloudMlClient()
    except Exception as e:
        err = "ERROR: Failed to init CloudML Client: {}".format(e)
        exit_as_error(err)
        return False, err

    response = client.delete_model_test_cases_by_org_id()

    print(response)
