import os
import sys
from cloud_ml_sdk.client import CloudMlClient


def exit_as_error(message, error_code=-1, is_os_exit=False):
  print(message)
  if is_os_exit:
    # os exit can not be catched
    os._exit(error_code)
  else:
    sys.exit(error_code)


def print_deploy_detail(response):
  for k, v in response.items():
    print("{:16} {}".format(k, v))
  return


def print_deploy_info_list(response):
  for r in response['data']:
    print("----------------------------------")
    for k, v in r.items():
      print("{:16} {}".format(k, v))
  return


def deploy_list(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return  
  response = client.list_deploy()
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_info_list(response)
  return response


def deploy_create(args):
  f = open(args.filename, "r")
  json_data = f.read()
  f.close()
  response = deploy_create_internal(json_data)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_create_internal(json_data):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return {"error": True, "message": "{}".format(e)}
  response = client.create_deploy(json_data)
  return response


def deploy_delete(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_info("delete", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_describe(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_info("describe", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_status(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_info("status", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_cmd(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_info("cmd", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_logs(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_info("logs", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print(response["logs"])
  return response


def deploy_event(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_info("event", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_stop(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("stop", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_rerun(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("rerun", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_pause(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("pause", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_resume(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("resume", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_abort(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("abort", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_finish(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("finish", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response


def deploy_setcmd_start(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.deploy_cmd("start", args.deploy_name, args.deploy_version)
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print_deploy_detail(response)
  return response
