#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 19-7-25 下午7:55
# @Author  : Bai Yun
# @File    : autoscaler_rule.py
# @Software: PyCharm Community Edition


class AutoScalerRule(object):
  def __init__(self, model_name, model_version, trigger_type):
    self.model_name = model_name
    self.model_version = model_version
    self.trigger_type = trigger_type
    self._instance_down_step = 1
    self._instance_up_step = 1
    self._collect_metrics_duration = 1
    self._trigger_interval = 10
    self._up_qps_buffer_percent = 0.001

  @property
  def model_name(self):
    return self._model_name

  @model_name.setter
  def model_name(self, value):
    if not isinstance(value, str):
      raise ValueError("model_name should be string.")
    self._model_name = value

  @property
  def model_version(self):
    return self._model_version

  @model_version.setter
  def model_version(self, value):
    try:
      self._model_version = int(value)
    except Exception as e:
      raise ValueError("model_version should be int:" + e)

  @property
  def trigger_type(self):
    return self._trigger_type

  @trigger_type.setter
  def trigger_type(self, value):
    if not isinstance(value, str):
      raise ValueError("trigger_type should be string.")
    self._trigger_type = value

  @property
  def up_threshold_qps(self):
    return self._up_threshold_qps

  @up_threshold_qps.setter
  def up_threshold_qps(self, value=2.0):
    try:
      self._up_threshold_qps = float(value)
    except Exception:
      raise ValueError("Parameters up_threshold_qps type error, must be float")

  @property
  def up_qps_buffer_percent(self):
    return self._up_qps_buffer_percent

  @up_qps_buffer_percent.setter
  def up_qps_buffer_percent(self, value=0.001):
    try:
      self._up_qps_buffer_percent = float(value)
    except Exception:
      raise ValueError("Parameters up_qps_buffer_percent type error, must be float.")

  @property
  def down_threshold_qps(self):
    return self._down_threshold_qps

  @down_threshold_qps.setter
  def down_threshold_qps(self, value=2.0):
    try:
      self._down_threshold_qps = float(value)
    except Exception:
      raise ValueError("Parameters down_threshold_qps type error, must be float")

  @property
  def threshold_max_replicas(self):
    return self._threshold_max_replicas

  @threshold_max_replicas.setter
  def threshold_max_replicas(self, value):
    try:
      self._threshold_max_replicas = int(value)
      if self._threshold_max_replicas < 1:
        raise ValueError("Parameters threshold_max_replicas must be gt 1.")
    except Exception:
      raise ValueError("Parameters threshold_max_replicas type error, must be int.")

  @property
  def threshold_min_replicas(self):
    return self._threshold_min_replicas

  @threshold_min_replicas.setter
  def threshold_min_replicas(self, value):
    try:
      self._threshold_min_replicas = int(value)
      if self._threshold_min_replicas < 1:
        raise ValueError("Parameters threshold_min_replicas must be gt 1.")
    except Exception:
      raise ValueError("Parameters threshold_min_replicas type error, must be int.")

  @property
  def instance_up_step(self):
    return self._instance_up_step

  @instance_up_step.setter
  def instance_up_step(self, value=1):
    try:
      self._instance_up_step = int(value)
      if self._instance_up_step < -1:
        raise ValueError("Parameters instance_up_step must be gt -1.")
    except Exception:
      raise ValueError("Parameters instance_up_step type error, must be int.")

  @property
  def instance_down_step(self):
    return self._instance_down_step

  @instance_down_step.setter
  def instance_down_step(self, value=1):
    try:
      self._instance_down_step = int(value)
      if self._instance_down_step < -1:
        raise ValueError("Parameters instance_down_step must be gt -1.")
    except Exception:
      raise ValueError("Parameters instance_down_step type error, must be int.")

  @property
  def collect_metrics_duration(self):
    return self._collect_metrics_duration

  @collect_metrics_duration.setter
  def collect_metrics_duration(self, value=1):
    try:
      self._collect_metrics_duration = int(value)
      if self._collect_metrics_duration < 1:
        raise ValueError("Parameters collect_metrics_duration must be gt 1.")
    except Exception:
      raise ValueError("Parameters collect_metrics_duration type error, must be int.")

  @property
  def trigger_interval(self):
    return self._trigger_interval

  @trigger_interval.setter
  def trigger_interval(self, value=10):
    try:
      self._trigger_interval = int(value)
      if self._trigger_interval < 1:
        raise ValueError("Parameters trigger_interval must be gt 1.")
    except Exception:
      raise ValueError("Parameters trigger_interval type error, must be int.")

  @property
  def falcon_endpoints(self):
    return self._falcon_endpoints

  @falcon_endpoints.setter
  def falcon_endpoints(self, value):
    try:
      self._falcon_endpoints = value
    except Exception:
      raise ValueError("falcon endpointss value error")

  @property
  def falcon_qps_counter_name(self):
    return self._falcon_qps_counter_name

  @falcon_qps_counter_name.setter
  def falcon_qps_counter_name(self, value):
    try:
      self._falcon_qps_counter_name = value
    except Exception:
      raise ValueError("falcon qps counter name value error")

  @property
  def cron_configs(self):
    return self._cron_configs

  @cron_configs.setter
  def cron_configs(self, value):
    try:
      self._cron_configs = value
    except Exception:
      raise ValueError("cron config value error")

  @property
  def pre_eco_time_second(self):
    return self._pre_eco_time_second

  @pre_eco_time_second.setter
  def pre_eco_time_second(self, value):
    try:
      self._pre_eco_time_second = value
    except Exception:
      raise ValueError("pre_eco_time_second value error")

  @property
  def eco_mode_threshold_qps(self):
    return self._eco_mode_threshold_qps

  @eco_mode_threshold_qps.setter
  def eco_mode_threshold_qps(self, value):
    try:
      self._eco_mode_threshold_qps = value
    except Exception:
      raise ValueError("eco_mode_threshold_qps value error")

  @property
  def falcon_fetcher_url_format(self):
    return self._falcon_fetcher_url_format

  @falcon_fetcher_url_format.setter
  def falcon_fetcher_url_format(self, value):
    try:
      if value == "":
        raise ValueError("falcon fetcher url format should not be empty")
      self._falcon_fetcher_url_format = value
    except Exception:
      raise ValueError("falcon fetcher url format value error")

  @property
  def falcon_fetcher_multi_model_tf_method_name(self):
    return self._falcon_fetcher_multi_model_tf_method_name

  @falcon_fetcher_multi_model_tf_method_name.setter
  def falcon_fetcher_multi_model_tf_method_name(self, value):
    try:
      self._falcon_fetcher_multi_model_tf_method_name = value
    except Exception:
      raise ValueError("falcon_fetcher_multi_model_tf_method_name value error")

  @property
  def falcon_fetcher_multi_model_tf_submodel_name(self):
    return self._falcon_fetcher_multi_model_tf_submodel_name

  @falcon_fetcher_multi_model_tf_submodel_name.setter
  def falcon_fetcher_multi_model_tf_submodel_name(self, value):
    try:
      self._falcon_fetcher_multi_model_tf_submodel_name = value
    except Exception:
      raise ValueError("falcon_fetcher_multi_model_tf_submodel_name value error")

  @property
  def config_json_file(self):
    return self._model_name

  @property
  def org_id(self):
    return self._org_id

  @org_id.setter
  def org_id(self, value):
    self._org_id = value

  def get_json_data(self):
    data = {}
    data["model_name"] = self.model_name
    data["model_version"] = self.model_version
    data["trigger_type"] = self.trigger_type
    data["threshold_max_replicas"] = self.threshold_max_replicas
    data["threshold_min_replicas"] = self.threshold_min_replicas
    data["instance_up_step"] = self.instance_up_step
    data["instance_down_step"] = self.instance_down_step
    data["collect_metrics_duration"] = self.collect_metrics_duration
    data["trigger_interval"] = self.trigger_interval
    if "qps" in self._trigger_type.split(","):
      if "_up_threshold_qps" not in self.__dict__:
        raise ValueError("If trigger type is qps, parameters up_threshold_qps must be set.")
      if "_down_threshold_qps" not in self.__dict__:
        raise ValueError("If trigger type is qps, parameters down_threshold_qps must be set.")
      data["up_threshold_qps"] = self.up_threshold_qps
      data["down_threshold_qps"] = self.down_threshold_qps
      data["up_qps_buffer_percent"] = self.up_qps_buffer_percent
      data["falcon_qps_counter_name"] = self.falcon_qps_counter_name
      data["falcon_endpoints"] = self.falcon_endpoints
      data["cron_configs"] = self.cron_configs
      data["pre_eco_time_second"] = self.pre_eco_time_second
      data["eco_mode_threshold_qps"] = self.eco_mode_threshold_qps
      data["falcon_fetcher_url_format"] = self.falcon_fetcher_url_format
      data["falcon_fetcher_multi_model_tf_method_name"] = self.falcon_fetcher_multi_model_tf_method_name
      data["falcon_fetcher_multi_model_tf_submodel_name"] = self.falcon_fetcher_multi_model_tf_submodel_name
    return data
