# Copyright 2018 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from . import models_util


class NamespaceQuotaService(object):
  """Class for namespace quota.
  """

  def __init__(self, namespace, cpu, memory, gpu, priority_class, org_mail, gpu_types, gpu_counts):
    self.namespace = namespace
    self.cpu = cpu
    self.memory = memory
    self.gpu = gpu
    self.priority_class = priority_class
    self.org_mail = org_mail
    if gpu_types:
      self.gpu_types = gpu_types
    if gpu_counts:
      self.gpu_counts = gpu_counts

  @property
  def namespace(self):
    return self._namespace

  @namespace.setter
  def namespace(self, value):
    if not isinstance(value, str):
      raise ValueError("namespace name must be a string!")
    if value == "":
      raise ValueError("namespace name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("namespace name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._namespace = value

  @property
  def priority_class(self):
    return self._priority_class

  @priority_class.setter
  def priority_class(self, value):
    if not value:
      raise ValueError("priority_class is required!")
    if not isinstance(value, str):
      raise ValueError("priority_class must be a string!")
    self._priority_class = value

  @property
  def cpu(self):
    return self._cpu

  @cpu.setter
  def cpu(self, value):
    self._cpu = value

  @property
  def memory(self):
    return self._memory

  @memory.setter
  def memory(self, value):
    self._memory = value

  @property
  def gpu(self):
    return self._gpu

  @gpu.setter
  def gpu(self, value):
    self._gpu = value

  @property
  def gpu_types(self):
    return self._gpu_types

  @gpu_types.setter
  def gpu_types(self, value):
    self._gpu_types = value

  @property
  def gpu_counts(self):
    return self._gpu_counts

  @gpu_counts.setter
  def gpu_counts(self, value):
    if len(value) != len(self.gpu_types):
      raise ValueError("gpu_counts per type must be same len as gpu_types!")
    self._gpu_counts = value

  @property
  def org_mail(self):
    return self._org_mail

  @org_mail.setter
  def org_mail(self, value):
    self._org_mail = value

  def get_json_data(self):
    data = {
      "namespace": self._namespace,
      "priority_class": self._priority_class,
    }
    if self._org_mail is not None:
      data["org_mail"] = self._org_mail
    if self._cpu is not None:
      data["cpu"] = self._cpu
    if self._memory is not None:
      data["memory"] = self._memory
    if self._gpu is not None:
      data["gpu"] = self._gpu
    if hasattr(self, "_gpu_types") and hasattr(self, "_gpu_counts"):
      data["gpus_per_type"] = dict(zip(self._gpu_types, self._gpu_counts))

    return json.dumps(data)
