# Copyright 2018 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from . import models_util


class PriorityClassService(object):
  """Class for priority_class.
  """

  def __init__(self, name, value, description):
    self.name = name
    self.value = value
    self.description = description

  @property
  def name(self):
    return self._name

  @name.setter
  def name(self, value):
    if not isinstance(value, str):
      raise ValueError("namespace name must be a string!")
    if value == "":
      raise ValueError("namespace name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("namespace name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._name = value

  @property
  def value(self):
    return self._value

  @value.setter
  def value(self, value):
    if value and not isinstance(value, int):
      raise ValueError("value must be a int!")
    self._value = value

  @property
  def description(self):
    return self._description

  @description.setter
  def description(self, value):
    if value and not isinstance(value, str):
      raise ValueError("description must be a string!")
    self._description = value


  def get_json_data(self):
    data = {
      "name": self._name,
      "value": self._value,
      "description": self._description,
    }

    return json.dumps(data)
