# Copyright 2017 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from . import models_util


class ProjectService(object):
  """Class for Project method.
  """

  def __init__(self, project_name, **kwargs):
    self.project_name = project_name

    self.description = kwargs.get("description", "")
    self.description = kwargs.get("description", "")
    self.code_scheme = kwargs.get("code_scheme", "")
    self.code_uri = kwargs.get("code_uri", "")
    self.code_secret = kwargs.get("code_secret", "")

  @property
  def project_name(self):
    return self._project_name

  @project_name.setter
  def project_name(self, value):
    if not isinstance(value, str):
      raise ValueError("project_name must be a string!")
    if value == "":
      raise ValueError("project_name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("project_name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._project_name = value

  @property
  def description(self):
    return self._description

  @description.setter
  def description(self, value):
    if not isinstance(value, str):
      raise ValueError("description must be a string!")
    self._description = value

  @property
  def code_scheme(self):
    return self._code_scheme

  @code_scheme.setter
  def code_scheme(self, value):
    if not isinstance(value, str):
      raise ValueError("code_scheme must be a string!")
    self._code_scheme = value

  @property
  def code_uri(self):
    return self._code_uri

  @code_uri.setter
  def code_uri(self, value):
    if not isinstance(value, str):
      raise ValueError("code_uri must be a string!")
    self._code_uri = value

  @property
  def code_secret(self):
    return self._code_secret

  @code_secret.setter
  def code_secret(self, value):
    if not isinstance(value, str):
      raise ValueError("code_secret must be a string!")
    self._code_secret = value

  def get_json_data(self):
    data = {
        "project_name": self._project_name,
        "description": self._description,
        "code": {
          "scheme": self._code_scheme,
          "uri": self._code_uri,
          "secret": self._code_secret,
        }
    }

    return json.dumps(data)
