# Copyright 2017 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from ..command import constant
from . import models_util


class SecretService(object):
  """Class for secret method.
  """

  def __init__(self, secret_name, data):
    self.secret_name = secret_name
    self.data = data

  @property
  def secret_name(self):
    return self._secret_name

  @secret_name.setter
  def secret_name(self, value):
    if not isinstance(value, str):
      raise ValueError("secret_name must be a string!")
    if value == "":
      raise ValueError("secret_name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("secret_name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._secret_name = value

  @property
  def data(self):
    return self._data

  @data.setter
  def data(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("data must be a string!")
    if value == "":
      raise ValueError("data cannot be Empty!")
    self._data = value

  def get_json_data(self):
    data = {
        "secret_name": self._secret_name,
        "data": self._data,
    }

    return json.dumps(data)


class HDFSSecretService(object):
  """Class for secret method.
  """
  def __init__(self, secret_name, keytab, password, endpoint=None, account=None):
    self.secret_name = secret_name
    if keytab:
      self.keytab = keytab
    if password:
      self.password = password
    if endpoint:
      self.endpoint = endpoint
    if account:
      self.account = account

  @property
  def secret_name(self):
    return self._secret_name

  @secret_name.setter
  def secret_name(self, value):
    if not isinstance(value, str):
      raise ValueError("secret_name must be a string!")
    if value == "":
      raise ValueError("secret_name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("secret_name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._secret_name = value

  @property
  def keytab(self):
    return self._keytab

  @keytab.setter
  def keytab(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("keytab must be a string!")
    if value == "":
      raise ValueError("keytab cannot be Empty!")
    self._keytab = value

  @property
  def password(self):
    return self._password

  @password.setter
  def password(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("password must be a string!")
    if value == "":
      raise ValueError("password cannot be Empty!")
    self._password = value

  @property
  def endpoint(self):
    return self._endpoint

  @endpoint.setter
  def endpoint(self, value):
    if not isinstance(value, str):
      raise ValueError("endpoint must be a string!")
    if value == "":
      raise ValueError("endpoint cannot be Empty!")
    self._endpoint = value

  @property
  def account(self):
    return self._account

  @account.setter
  def account(self, value):
    if not isinstance(value, str):
      raise ValueError("account must be a string!")
    if value == "":
      raise ValueError("account cannot be Empty!")
    self._account = value

  def get_json_data(self):
    data = {
        "secret_name": self._secret_name,
    }
    if self._keytab:
      data["keytab"] = self._keytab
    if self._password:
      data["password"] = self._password
    if self._endpoint:
      data["endpoint"] = self._endpoint
    if self._account:
      data["account"] = self._account

    return json.dumps(data)


class FDSSecretService(object):
  """Class for secret method.
  """
  def __init__(self, secret_name, access_key, secret_key, endpoint=None, bucket=None):
    self.secret_name = secret_name
    self.access_key = access_key
    self.secret_key = secret_key
    if endpoint:
      self.endpoint = endpoint
    if bucket:
      self.bucket = bucket

  @property
  def secret_name(self):
    return self._secret_name

  @secret_name.setter
  def secret_name(self, value):
    if not isinstance(value, str):
      raise ValueError("secret_name must be a string!")
    if value == "":
      raise ValueError("secret_name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("secret_name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._secret_name = value

  @property
  def access_key(self):
    return self._access_key

  @access_key.setter
  def access_key(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("access_key must be a string!")
    if value == "":
      raise ValueError("access_key cannot be Empty!")
    self._access_key = value

  @property
  def secret_key(self):
    return self._secret_key

  @secret_key.setter
  def secret_key(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("secret_key must be a string!")
    if value == "":
      raise ValueError("secret_key cannot be Empty!")
    self._secret_key = value

  @property
  def endpoint(self):
    return self._endpoint

  @endpoint.setter
  def endpoint(self, value):
    if not isinstance(value, str):
      raise ValueError("endpoint must be a string!")
    if value == "":
      raise ValueError("endpoint cannot be Empty!")
    self._endpoint = value

  @property
  def bucket(self):
    return self._bucket

  @bucket.setter
  def bucket(self, value):
    if not isinstance(value, str):
      raise ValueError("bucket must be a string!")
    if value == "":
      raise ValueError("bucket cannot be Empty!")
    self._bucket = value

  def get_json_data(self):
    data = {
      "secret_name": self._secret_name,
      "access_key": self._access_key,
      "secret_key": self._secret_key,
    }
    if self._endpoint:
      data["endpoint"] = self._endpoint
    if self._bucket:
      data["bucket"] = self._bucket

    return json.dumps(data)


class GitSecretService(object):
  """Class for secret method.
  """
  def __init__(self, secret_name, ssh, known_hosts):
    self.secret_name = secret_name
    self.ssh = ssh
    self.known_hosts = known_hosts

  @property
  def secret_name(self):
    return self._secret_name

  @secret_name.setter
  def secret_name(self, value):
    if not isinstance(value, str):
      raise ValueError("secret_name must be a string!")
    if value == "":
      raise ValueError("secret_name cannot be None!")
    if not models_util.check_kube_resource_name_regex(value):
      raise Exception("secret_name must match {}.".format(
          models_util.kube_resource_name_regex))
    self._secret_name = value

  @property
  def ssh(self):
    return self._ssh

  @ssh.setter
  def ssh(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("ssh must be a string!")
    if value == "":
      raise ValueError("ssh cannot be Empty!")
    self._ssh = value

  @property
  def known_hosts(self):
    return self._known_hosts

  @known_hosts.setter
  def known_hosts(self, value):
    if isinstance(value, bytes):
      value = str(value.decode("utf-8"))
    if not isinstance(value, str):
      raise ValueError("known_hosts must be a string!")
    if value == "":
      raise ValueError("known_hosts cannot be Empty!")
    self._known_hosts = value

  def get_json_data(self):
    data = {
      "secret_name": self._secret_name,
      "ssh": self._ssh,
      "known_hosts": self._known_hosts,
    }

    return json.dumps(data)
