import os
import json
import sys
import hashlib
import cloud_ml_sdk.command.util
from cloud_ml_sdk.command import constant

  
def list_dataset(args):
  print("hello list_dataset")
  client = cloud_ml_sdk.command.util.get_client()
  data = {}
  if args.access_key:
    data['access_key'] = args.access_key
  else:
    config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
    config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)
    config_data = {}
    if os.path.exists(config_path):
      try:
        with open(config_path) as data_file:
          config_data = json.load(data_file)
      except Exception as e:
        print(
            "{} already exists. But failed to load config file, exception: {}".format(
                config_path, e))
        exit_as_error("You need to fix the config file by hand.")
        return
    data['access_key'] = config_data['xiaomi_access_key_id']
  print(client.list_dataset(json.dumps(data)))

def exist_dataset(args):
  client = cloud_ml_sdk.command.util.get_client()
  data = {}
  data["dataset_name"] = args.dataset_name
  data["version"] = args.version
  if False:
    data['access_key'] = args.access_key
  else:
    config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
    config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)
    config_data = {}
    if os.path.exists(config_path):
      try:
        with open(config_path) as data_file:
          config_data = json.load(data_file)
      except Exception as e:
        print(
            "{} already exists. But failed to load config file, exception: {}".format(
                config_path, e))
        exit_as_error("You need to fix the config file by hand.")
        return
    data['access_key'] = config_data['xiaomi_access_key_id']
  print(client.exist_dataset(json.dumps(data)))


def create_dataset(args):
  print("hello create_dataset")
  client = cloud_ml_sdk.command.util.get_client()
  data = {}
  data["dataset_name"] = args.dataset_name
  data["type"] = args.type
  data["version"] = args.version
  data["filepath"] = args.filepath

  if False: #args.access_key:
    data['access_key'] = args.access_key
  else:
    config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
    config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)
    config_data = {}
    if os.path.exists(config_path):
      try:
        with open(config_path) as data_file:
          config_data = json.load(data_file)
      except Exception as e:
        print(
            "{} already exists. But failed to load config file, exception: {}".format(
                config_path, e))
        exit_as_error("You need to fix the config file by hand.")
        return
    data['access_key'] = config_data['xiaomi_access_key_id']

  client.create_dataset(json.dumps(data))


def download_dataset(args):

  client = cloud_ml_sdk.command.util.get_client()
  data = {}
  data["dataset_name"] = args.dataset_name
  data["version"] = args.version
  data["filepath"] = args.filepath
  data["type"] = args.type
  data["label_name"] = args.label_name

  if False: # args.access_key:
    data['access_key'] = args.access_key
  else:
    config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
    config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)
    config_data = {}
    if os.path.exists(config_path):
      try:
        with open(config_path) as data_file:
          config_data = json.load(data_file)
      except Exception as e:
        print(
            "{} already exists. But failed to load config file, exception: {}".format(
                config_path, e))
        exit_as_error("You need to fix the config file by hand.")
        return
    data['access_key'] = config_data['xiaomi_access_key_id']
    data['bucket'] = config_data['cloudml_default_fds_bucket']
  
  print(client.download_dataset(json.dumps(data)))


def label_dataset(args):
  client = cloud_ml_sdk.command.util.get_client()
  data = {}
  data['label_name'] = args.label_name
  data['type'] = args.type
  if args.type == 'delete':
    data['filemd5'] = ""
  else:
    data['filemd5'] = getMD5File(args.filepath)

  if False: #args.access_key:
    data['access_key'] = args.access_key
  else:
    config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
    config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)
    config_data = {}
    if os.path.exists(config_path):
      try:
        with open(config_path) as data_file:
          config_data = json.load(data_file)
      except Exception as e:
        print(
            "{} already exists. But failed to load config file, exception: {}".format(
                config_path, e))
        exit_as_error("You need to fix the config file by hand.")
        return
    data['access_key'] = config_data['xiaomi_access_key_id']

  print(client.label_dataset(json.dumps(data)))


def getMD5File(filepath):
  reload(sys)
  sys.setdefaultencoding('utf-8')
  with open(filepath, 'rb') as fp:
      data = fp.read()
  file_md5= hashlib.md5(data).hexdigest()
  return file_md5


def share_dataset(args):
  client = cloud_ml_sdk.command.util.get_client()
  data = {}
  data['dataset_name'] = args.dataset_name
  data['org_id'] = args.org_id
  data['type'] = args.type
  if False: #args.access_key:
    data['access_key'] = args.access_key
  else:
    config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
    config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)
    config_data = {}
    if os.path.exists(config_path):
      try:
        with open(config_path) as data_file:
          config_data = json.load(data_file)
      except Exception as e:
        print(
            "{} already exists. But failed to load config file, exception: {}".format(
                config_path, e))
        exit_as_error("You need to fix the config file by hand.")
        return
    data['access_key'] = config_data['xiaomi_access_key_id']
  print(client.share_dataset(json.dumps(data)))

