#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 19-11-11 下午5:01
# @Author  : Bai Yun
# @File    : load_balance_dns.py
# @Software: PyCharm Community Edition
from threading import Thread, Lock
from dns import message, query
import time
import logging

from cloud_ml_sdk.utils.singleton_util import synchronized, Singleton

logging.basicConfig(level=logging.INFO, format ='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)


class ChannelStore(object):
  def __init__(self, func, options, compression, port=9000):
    self.__store_key = []
    self.__store_value = []
    self.__index = 0
    self.__func = func
    self.__options = options
    self.__compression = compression
    self.__port = port

  @synchronized
  def refresh(self, ips):
    for ip in ips:
      if ip not in self.__store_key:
        self.__store_key.append(ip)
        self.__store_value.append(self.__func("{}:{}".format(ip, self.__port)))
    for ind, ip in enumerate(self.__store_key):
      if ip not in ips:
        self.__store_key.pop(ind)
        self.__store_value.pop(ind)

  @synchronized
  def channel(self):
    channel = self.__store_value[self.__index]
    logger.info("request instance ip is {}".format(self.__store_key[self.__index]))
    self.__index += 1
    if self.__index == len(self.__store_value):
      self.__index = 0
    return channel

  @synchronized
  def put(self, key, value):
    if key not in self.__store_key:
      self.__store_key.append(key)
      self.__store_value.append(value)

  @synchronized
  def exists(self, key):
    if key in self.__store_key:
      return True
    else:
      return False

  @synchronized
  def delete(self, key):
    for ind, ip in enumerate(self.__store_key):
      if ip == key:
        self.__store_key.pop(ind)
        self.__store_value.pop(ind)
    return True


@Singleton
class DnsLoadBanlance(object):
  def __init__(self, dns_ip, grpc_address, port, func, options=None, compression=None, refresh=3):
    self.dns_ip = dns_ip
    self.grpc_address = grpc_address
    self.port = port
    self.__channel_store = ChannelStore(func, options, compression)
    self.__refresh()
    ref_th = Thread(target=self.__timer_refresh, args=(refresh, ))
    ref_th.setDaemon(True)
    ref_th.start()

  def __dns_resolve(self, dns_ip, address):
    ip_list = []
    try:
      query_A = message.make_query(address, 'A')
      response = query.udp(query_A, dns_ip, 53)
      for i in response.answer:
        for j in i.items:
          ip_list.append(j.address)
    except Exception as e:
      logger.warning("Resolver address {} error, exception: {}".format(address, e))
    finally:
      return ip_list

  def __refresh(self):
    try:
      ips = self.__dns_resolve(self.dns_ip, self.grpc_address)
      self.__channel_store.refresh(ips)
    except Exception as e:
      logger.error("refresh exception: {}".format(e))

  def __timer_refresh(self, refresh):
    while True:
      time.sleep(refresh)
      self.__refresh()

  def channel(self):
    return self.__channel_store.channel()


def request_serving(load_banlance, stub_class, stub_method, request_data, retry=3, **kwargs):
  for i in xrange(retry):
    try:
      stub = stub_class(load_banlance.channel())
      response = stub.__getattribute__(stub_method)(request_data, **kwargs)
      return response
    except Exception as e:
      logger.info("wait retry, request exception : {}".format(e.message))
