import json
import logging

import requests


class settings:
  ES_URL = 'http://zjysrv.api.es.srv'
  ENABLE_ES_LOG = True
  ES_LOG_TABLE = 'aiservice_aiplatform_cloudml_c5_k8s_log'

def get_log_from_es(pod_name, tail_lines=None):
  if not settings.ENABLE_ES_LOG:
    return ''
  scroll_time = "1m"
  timeout = 50
  init_scroll_url = settings.ES_URL +   "/" + settings.ES_LOG_TABLE + "-*/_search?scroll=" + scroll_time
  init_data = {
    "_source": [
      "log"
    ],
    "size": 10000,
    "query": {
      "match": {
        "kubernetes.pod_name": {
          "query": pod_name
        }
      }
     }
    ,
    "sort": [
      {
        "logtime": {
          "order": "desc"
        }
      }
    ]
  }
  headers = {"Content-Type": "application/json"}
  ret = []
  parsed_tail_lines = 99999999 if tail_lines is None else int(tail_lines)
  parsed_ret = []
  try:
    init_response = requests.get(url=init_scroll_url, data=json.dumps(init_data), headers=headers, timeout=timeout)
    rep_data = json.loads(init_response.content)
    # print rep_data
    if "_scroll_id" not in rep_data:
      raise Exception('There is no pod[{}] log in es'.format(pod_name))
    scroll_id = rep_data['_scroll_id']
    cur_data = []
    if 'hits' in rep_data and 'hits' in rep_data['hits']:
      cur_data = rep_data['hits']['hits']
    ret = ret + cur_data
    while len(cur_data) > 0 and len(ret) < parsed_tail_lines:
      next_url = settings.ES_URL + "/_search/scroll"
      scroll_param = {"scroll_id": scroll_id, "scroll": scroll_time}
      response = requests.get(url=next_url, data=json.dumps(scroll_param), headers=headers, timeout=timeout)
      rep_data = json.loads(response.content)
      if 'hits' in rep_data and 'hits' in rep_data['hits']:
        cur_data = rep_data['hits']['hits']
      else:
        cur_data = []
      ret = ret + cur_data

    for x in ret:
      parsed_ret.append(x['_source']['log'])
  except Exception as e:
    # add logs and do nothing
    logging.error("Error when get {} logs from es: {}".format(pod_name, e))
  if tail_lines:
    parsed_ret = parsed_ret[0:parsed_tail_lines]
  return "".join(reversed(parsed_ret))