from threading import Lock

def synchronized(func):
  func.__lock__ = Lock()
  def synced_func(*args, **kwargs):
    func.__lock__.acquire()
    tmp = func(*args, **kwargs)
    func.__lock__.release()
    return tmp
  return synced_func


def Singleton(cls):
  instances = {}
  @synchronized
  def get_instance(*args, **kwargs):
    if cls not in instances:
      instances[cls] = cls(*args, **kwargs)
    return instances[cls]
  return get_instance